/**
 * This file is part of spy4js which is released under MIT license.
 *
 * The LICENSE file can be found in the root directory of this project.
 *
 */
import { COMPARE, MAPPER, type OptionalMessageOrError, type MessageOrError } from './utils';
import { type Mockable } from './mock';
import { Symbols } from './symbols';
import { configure, configureAll } from './config';
type SpyInstanceConfig = {
    useOwnEquals: boolean;
    persistent: boolean;
};
type FlatAwaited<TVal> = TVal extends PromiseLike<infer TAwaited> ? TAwaited : never;
export type SpyInstance<TFunc extends (...args: any) => any = (...args: any) => any> = TFunc & {
    configure: (config: Partial<SpyInstanceConfig>) => SpyInstance<TFunc>;
    calls: (...funcs: TFunc[]) => SpyInstance<TFunc>;
    returns: (...args: ReturnType<TFunc>[]) => SpyInstance<TFunc>;
    resolves: FlatAwaited<ReturnType<TFunc>> extends never ? never : (...args: FlatAwaited<ReturnType<TFunc>>[]) => SpyInstance<TFunc>;
    rejects: FlatAwaited<ReturnType<TFunc>> extends never ? never : (...msgOrErrors: OptionalMessageOrError[]) => SpyInstance<TFunc>;
    throws: (msgOrError?: MessageOrError) => SpyInstance<TFunc>;
    reset: () => SpyInstance<TFunc>;
    restore: () => SpyInstance<TFunc>;
    transparent: () => SpyInstance<TFunc>;
    transparentAfter: (callCount: number) => SpyInstance<TFunc>;
    addSnapshotSerializer: (serialize: string | ((...args: any[]) => string)) => SpyInstance<TFunc>;
    wasCalled: (callCount?: number) => void;
    hasCallHistory: (...callHistory: Array<any[] | any>) => void;
    wasNotCalled: () => void;
    wasCalledWith: (...args: any[]) => void;
    wasNotCalledWith: (...args: any[]) => void;
    getAllCallArguments: () => Array<any[]>;
    getCallArguments: (callNr?: number) => any[];
    getCallArgument: (callNr?: number, argNr?: number) => any;
    getLatestCallArgument: (argNr?: number) => any;
    getProps: () => any;
    hasProps: (props: any) => void;
    getCallCount: () => number;
    showCallArguments: (additionalInformation?: (string | undefined)[]) => string;
    [Symbols.name]: string;
    [Symbols.snap]: string;
    [Symbols.snapSerializer]?: (...args: any[]) => string;
    [Symbols.calls]: {
        args: any[];
        order: number;
    }[];
    [Symbols.index]: number;
    [Symbols.isSpy]: boolean;
    [Symbols.func]: Function;
    [Symbols.config]: SpyInstanceConfig;
    [Symbols.onRestore]?: () => void;
};
type ISpy = {
    (name?: string): SpyInstance;
    configure: typeof configure;
    setup: typeof configureAll;
    IGNORE: Symbol;
    COMPARE: typeof COMPARE;
    MAPPER: typeof MAPPER;
    on<T extends Mockable, K extends keyof T>(obj: T, methodName: K): SpyInstance<T[K]>;
    mock<T extends Mockable, K extends keyof T>(obj: T, ...methodNames: K[]): {
        [P in K]: SpyInstance<T[P]>;
    };
    mockReactComponents<T extends Mockable, K extends keyof T>(obj: T, ...methodNames: K[]): {
        [P in K]: SpyInstance<T[P]>;
    };
    restoreAll(): void;
    resetAll(): void;
};
/**
 * This constructor does instantiate a Spy
 * object.
 *
 * This spy is callable.
 * It does inherit all Spy specific methods below.
 * It holds additional (private) fields:
 * _name:string -> Will be displayed in all displayed
 *                 error messages.
 * _isSpy:boolean -> Always true for spies.
 * _func:Function -> The internal function, that will
 *                   actually we called, when calling
 *                   the spy.
 * _calls:Array<{arguments:Array<any>}>
 *     -> Stores the arguments with whom the spy was called.
 *        Each call adds another entry in the calls array.
 * _config = {useOwnEquals: boolean} -> internal spy config.
 *
 *
 * @param {string} name -> the identifier of the spy.
 *                       Useful for debugging issues.
 */
declare const Spy: ISpy;
export { Spy };
